<?php

namespace FiloBlu\Rma\Model\Rma\Status;

use FiloBlu\Rma\Model\RmaEmailManagment;
use Magento\Framework\Api\AttributeValueFactory;
use Magento\Framework\Exception\MailException;
use Magento\Framework\Stdlib\DateTime\TimezoneInterface;
use Magento\Rma\Api\RmaAttributesManagementInterface;
use Magento\Rma\Api\RmaRepositoryInterface;
use Magento\Sales\Model\Order\Address\Renderer as AddressRenderer;

/**
 * Class History
 * @package Model\Rma\Status
 *
 */
class History extends \Magento\Rma\Model\Rma\Status\History
{
    /**
     * @var string
     */
    const XML_PATH_RECEIVED_EMAIL = 'sales_email/magento_rma_received';

    /**
     * @var RmaEmailManagment
     */
    protected $rmaEmailManagment;


    /**
     * History constructor.
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\Api\ExtensionAttributesFactory $extensionFactory
     * @param AttributeValueFactory $customAttributeFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Rma\Model\RmaFactory $rmaFactory
     * @param \Magento\Rma\Model\Config $rmaConfig
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $dateTimeDateTime
     * @param \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation
     * @param \Magento\Rma\Helper\Data $rmaHelper
     * @param TimezoneInterface $localeDate
     * @param RmaRepositoryInterface $rmaRepositoryInterface
     * @param RmaAttributesManagementInterface $metadataService
     * @param AddressRenderer $addressRenderer
     * @param RmaEmailManagment $rmaEmailManagment
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource|null $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(\Magento\Framework\Model\Context                        $context,
                                \Magento\Framework\Registry                             $registry,
                                \Magento\Framework\Api\ExtensionAttributesFactory       $extensionFactory,
                                AttributeValueFactory                                   $customAttributeFactory,
                                \Magento\Store\Model\StoreManagerInterface              $storeManager,
                                \Magento\Rma\Model\RmaFactory                           $rmaFactory,
                                \Magento\Rma\Model\Config                               $rmaConfig,
                                \Magento\Framework\Mail\Template\TransportBuilder       $transportBuilder,
                                \Magento\Framework\Stdlib\DateTime\DateTime             $dateTimeDateTime,
                                \Magento\Framework\Translate\Inline\StateInterface      $inlineTranslation,
                                \Magento\Rma\Helper\Data                                $rmaHelper,
                                TimezoneInterface                                       $localeDate,
                                RmaRepositoryInterface                                  $rmaRepositoryInterface,
                                RmaAttributesManagementInterface                        $metadataService,
                                AddressRenderer                                         $addressRenderer,
                                RmaEmailManagment                                       $rmaEmailManagment,
                                \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
                                \Magento\Framework\Data\Collection\AbstractDb           $resourceCollection = null,
                                array                                                   $data = [])
    {
        $this->rmaEmailManagment = $rmaEmailManagment;
        parent::__construct($context, $registry, $extensionFactory, $customAttributeFactory, $storeManager, $rmaFactory, $rmaConfig, $transportBuilder, $dateTimeDateTime, $inlineTranslation, $rmaHelper, $localeDate, $rmaRepositoryInterface, $metadataService, $addressRenderer, $resource, $resourceCollection, $data);
    }


    /**
     * @param $rma
     * @param $rootConfig
     * @return $this
     */
    protected function sendEmail($rma , $rootConfig)
    {
        try {
            $storeId = $rma->getStoreId();
            $order = $rma->getOrder();

            $this->_rmaConfig->init($rootConfig, $storeId);
            if (!$this->_rmaConfig->isEnabled()) {
                return $this;
            }

            $this->inlineTranslation->suspend();

            $copyTo = $this->_rmaConfig->getCopyTo();
            $copyMethod = $this->_rmaConfig->getCopyMethod();

            if ($order->getCustomerIsGuest()) {
                $template = $this->_rmaConfig->getGuestTemplate();
                $customerName = $order->getBillingAddress()->getName();
            } else {
                $template = $this->_rmaConfig->getTemplate();
                $customerName = $rma->getCustomerName();
            }

            $sendTo = [['email' => $order->getCustomerEmail(), 'name' => $customerName]];
            if ($rma->getCustomerCustomEmail() && $rma->getCustomerCustomEmail() !== $order->getCustomerEmail()) {
                $sendTo[] = ['email' => $rma->getCustomerCustomEmail(), 'name' => $customerName];
            }
            if ($copyTo && $copyMethod == 'copy') {
                foreach ($copyTo as $email) {
                    $sendTo[] = ['email' => $email, 'name' => null];
                }
            }

            $returnAddress = $this->rmaHelper->getReturnAddress('html', [], $storeId);

            $bcc = [];
            if ($copyTo && $copyMethod == 'bcc') {
                $bcc = $copyTo;
            }
            $store = $this->_storeManager->getStore($storeId);
            $identity = $this->_rmaConfig->getIdentity('', $storeId);


            foreach ($sendTo as $recipient) {
                $transport = $this->_transportBuilder->setTemplateIdentifier($template)
                    ->setTemplateOptions(['area' => \Magento\Framework\App\Area::AREA_FRONTEND, 'store' => $storeId])
                    ->setTemplateVars(
                        [
                            'rma' => $rma,
                            'easy_return' => $rma->getIsEasyReturn(),
                            'rma_id' => $rma->getId(),
                            'rma_data' => [
                                'status_label' => is_string($rma->getStatusLabel()) ?
                                    $rma->getStatusLabel() : $rma->getStatusLabel()->render(),
                            ],
                            'order' => $order,
                            'order_data' => [
                                'customer_name' => $order->getCustomerName(),
                            ],
                            'created_at_formatted_1' => $rma->getCreatedAtFormated(1),
                            'store' => $store,
                            'return_address' => $returnAddress,
                            'item_collection' => $rma->getItemsForDisplay(),
                            'formattedShippingAddress' => $this->addressRenderer->format(
                                $order->getShippingAddress(),
                                'html'
                            ),
                            'formattedBillingAddress' => $this->addressRenderer->format(
                                $order->getBillingAddress(),
                                'html'
                            ),
                            'supportEmail' => $store->getConfig('trans_email/ident_support/email'),
                            'storePhone' => $store->getConfig('general/store_information/phone'),
                        ]
                    )
                    ->setFromByScope($identity, $storeId)
                    ->addTo($recipient['email'], $recipient['name'])
                    ->addBcc($bcc)
                    ->getTransport();

                $transport->sendMessage();
            }

            $this->setEmailSent(true);
        } catch (MailException $exception) {
            $this->_logger->critical($exception->getMessage());
        } catch (\Exception $e){
            $this->_logger->critical($e->getMessage());
        }catch (\Throwable $t){
            $this->_logger->critical($t->getMessage());
        }

        $this->inlineTranslation->resume();

        return $this;
    }


    /**
     * @return History|\Magento\Rma\Model\Rma\Status\History
     */
    public function sendReceivedEmail()
    {
        try {
            return $this->_sendRmaEmailWithItems($this->getRma(), $this->getRootReceivedRmaEmail());
        } catch (\Exception $e) {
            $this->_logger->critical($e->getMessage());
        }
    }

    /**
     * @param $config
     * @return History|\Magento\Rma\Model\Rma\Status\History
     */
    public function sendRmaEmail($config)
    {
        try {
            return $this->sendEmail($this->getRma(), $config);
           // return $this->_sendRmaEmailWithItems($this->getRma(), $config);
        } catch (\Exception $e) {
            $this->_logger->critical($e->getMessage());
        }

    }

    /**
     * @return string
     */
    public function getRootReceivedRmaEmail()
    {
        return self::XML_PATH_RECEIVED_EMAIL;
    }

    /**
     * @return History|\Magento\Rma\Model\Rma\Status\History
     */
    public function sendAuthorizeEmail()
    {
        $rma = $this->getRma();
        return $this->rmaEmailManagment->checkIfSendRmaEmail($rma);
    }

    /**
     * Sending email with RMA data
     *
     * @return $this
     */
    public function sendNewRmaEmail()
    {
        return $this;
    }


}
